@@ -20,9 +20,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <glib.h>
+#include <glib-object.h>
+#include <glib/gprintf.h>
 #include "jawutil.h"
 #include "jawimpl.h"
 #include "jawtoplevel.h"
+#include "jawobject.h"
 
 static void jaw_impl_class_init (JawImplClass *klass);
 //static void			jaw_impl_init				(JawImpl		*impl);
@@ -82,20 +85,6 @@ static gpointer jaw_impl_parent_class = NULL;
 
 static GHashTable *typeTable = NULL;
 static GHashTable *objectTable = NULL;
-static GMutex *objectTableMutex = NULL;
-
-typedef struct {
-  GMutex jg_mutex;
-} JGMutex;
-
-JGMutex *jmutex;
-
-void
-jaw_impl_init_mutex ()
-{
-  jmutex = g_new(JGMutex, 1);
-  g_mutex_init (&jmutex->jg_mutex);
-}
 
 static void
 object_table_insert (JNIEnv *jniEnv, jobject ac, JawImpl * jaw_impl)
@@ -107,10 +96,7 @@ object_table_insert (JNIEnv *jniEnv, jobject ac, JawImpl * jaw_impl)
                                           "hashCode",
                                           "()I");
   gint hash_key = (gint)(*jniEnv)->CallIntMethod(jniEnv, ac, jmid);
-
-  g_mutex_lock(objectTableMutex);
   g_hash_table_insert(objectTable, (gpointer)&hash_key, (gpointer)jaw_impl);
-  g_mutex_unlock(objectTableMutex);
 }
 
 static JawImpl*
@@ -125,10 +111,7 @@ object_table_lookup (JNIEnv *jniEnv, jobject ac)
   gint hash_key = (gint)(*jniEnv)->CallIntMethod( jniEnv, ac, jmid );
   gpointer value = NULL;
 
-  g_mutex_lock(objectTableMutex);
   value = g_hash_table_lookup(objectTable, (gpointer)&hash_key);
-  g_mutex_unlock(objectTableMutex);
-
   return (JawImpl*)value;
 }
 
@@ -143,9 +126,7 @@ object_table_remove(JNIEnv *jniEnv, jobject ac)
                                           "()I" );
   gint hash_key = (gint)(*jniEnv)->CallIntMethod( jniEnv, ac, jmid );
 
-  g_mutex_lock(objectTableMutex);
-  g_hash_table_remove( objectTable, (gpointer)&hash_key );
-  g_mutex_unlock(objectTableMutex);
+  g_hash_table_remove(objectTable, (gpointer)&hash_key );
 }
 
 static void
@@ -251,32 +232,50 @@ JawImpl*
 jaw_impl_get_instance (JNIEnv *jniEnv, jobject ac)
 {
   JawImpl *jaw_impl;
-
-  g_mutex_lock(objectTableMutex);
-  if (objectTable == NULL) 
+  jniEnv = jaw_util_get_jni_env();
+  if (objectTable == NULL)
   {
     objectTable = g_hash_table_new ( NULL, NULL );
   }
-  g_mutex_unlock(objectTableMutex);
 
   jaw_impl = object_table_lookup( jniEnv, ac );
 
   if (jaw_impl == NULL)
   {
     jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, ac);
-    guint tflag = jaw_util_get_tflag_from_jobj(jniEnv, global_ac);
-    jaw_impl = g_object_new( JAW_TYPE_IMPL(tflag), NULL );
-    JawObject *jaw_obj = JAW_OBJECT(jaw_impl);
-    jaw_obj->acc_context = global_ac;
-    jaw_obj->storedData = g_hash_table_new(g_str_hash, g_str_equal);
-    aggregate_interface(jniEnv, jaw_obj, tflag);
+    if (global_ac != NULL)
+    {
+      guint tflag = jaw_util_get_tflag_from_jobj(jniEnv, global_ac);
+      jaw_impl = g_object_new( JAW_TYPE_IMPL(tflag), NULL );
+      if (jaw_impl != NULL)
+      {
+        JawObject *jaw_obj = JAW_OBJECT(jaw_impl);
 
-    atk_object_initialize( ATK_OBJECT(jaw_impl), NULL );
-
-    object_table_insert( jniEnv, global_ac, jaw_impl);
+        if (jaw_obj != NULL)
+        {
+          jaw_obj->acc_context = global_ac;
+          jaw_obj->storedData = g_hash_table_new(g_str_hash, g_str_equal);
+          aggregate_interface(jniEnv, jaw_obj, tflag);
+          atk_object_initialize( ATK_OBJECT(jaw_impl), NULL );
+          object_table_insert( jniEnv, global_ac, jaw_impl);
+        } else {
+          printf("\n *** JAW_OBJECT == NULL *** jaw_impl_get_instance: %s \n",
+                 (char *) (ATK_OBJECT(jaw_obj)));
+        }
+      } else {
+        printf("\n *** JAW_IMPL == NULL ***  jaw_impl_get_instance: %s \n",
+               (char *) ATK_OBJECT(jaw_impl));
+      }
+    }
   }
-
-	return jaw_impl;
+  if (jaw_impl != NULL)
+  {
+    return jaw_impl;
+  }
+  else {
+    printf("\n *** JAW_IMPL == NULL *** jaw_impl_get_instance\n");
+  }
+  return NULL;
 }
 
 JawImpl*
@@ -284,16 +283,18 @@ jaw_impl_find_instance (JNIEnv *jniEnv, jobject ac)
 {
   JawImpl *jaw_impl;
 
-  g_mutex_lock(objectTableMutex);
   if (objectTable == NULL)
-  {
-    g_mutex_unlock(objectTableMutex);
-    return NULL;
-  }
-  g_mutex_unlock(objectTableMutex);
+   return NULL;
 
   jaw_impl = object_table_lookup( jniEnv, ac );
 
+  if (jaw_impl == NULL)
+  {
+    printf("\n *** JAW_IMPL == NULL ***  jaw_impl_find_instance: %s \n",
+           (char *) ATK_OBJECT(jaw_impl));
+    return NULL;
+  }
+
   return jaw_impl;
 }
 
@@ -621,7 +622,8 @@ jaw_impl_ref_child (AtkObject *atk_obj, gint i)
   jobject child_ac = (*jniEnv)->CallObjectMethod( jniEnv, jchild, jmid );
 
   AtkObject *obj = (AtkObject*) jaw_impl_get_instance( jniEnv, child_ac );
-  g_object_ref (G_OBJECT(obj));
+  if (G_OBJECT(obj) != NULL)
+    g_object_ref (G_OBJECT(obj));
 
   return obj;
 }
@@ -778,8 +780,8 @@ jaw_impl_ref_relation_set (AtkObject *atk_obj)
                                       "()Ljavax/accessibility/AccessibleContext;");
         jobject target_ac = (*jniEnv)->CallObjectMethod( jniEnv, jtarget, jmid );
 
-        AtkObject *target_obj = (AtkObject*)jaw_impl_get_instance( jniEnv, target_ac );
-        atk_object_add_relationship(atk_obj, rel_type, target_obj);
+        JawImpl *target_obj = jaw_impl_get_instance( jniEnv, target_ac );
+        atk_object_add_relationship(atk_obj, rel_type, (AtkObject*) target_obj);
       }
     }
   }
