@@ -24,8 +24,10 @@ import java.awt.event.*;
 import java.beans.*;
 import java.io.*;
 import javax.accessibility.*;
+import java.awt.Toolkit;
 
 public class AtkWrapper {
+	public native void AtkWrapper();
 	static boolean accessibilityEnabled = false;
 	static {
 		try {
@@ -47,6 +49,7 @@ public class AtkWrapper {
 			}
 		} catch (Exception e) {
 			e.printStackTrace();
+			e.getCause();
 		}
 	}
 
@@ -61,7 +64,7 @@ public class AtkWrapper {
 
 		public void windowDeactivated(WindowEvent e) {
 			Object o = e.getSource();
-			if ( o instanceof javax.accessibility.Accessible ) {
+			if ( o instanceof javax.accessibility.Accessible &  ((e.getNewState() & Frame.MAXIMIZED_BOTH) == Frame.MAXIMIZED_BOTH) ) {
 				AccessibleContext ac = ((javax.accessibility.Accessible)o).getAccessibleContext();
 				AtkWrapper.windowDeactivate(ac);
 			}
@@ -175,10 +178,14 @@ public class AtkWrapper {
 					}
 					case ContainerEvent.COMPONENT_REMOVED:
 					{
-						java.awt.Component c = ((ContainerEvent)e).getChild();
-						if (c instanceof javax.accessibility.Accessible) {
-							AccessibleContext ac = ((javax.accessibility.Accessible)c).getAccessibleContext();
-							AtkWrapper.componentRemoved(ac);
+						try {
+							java.awt.Component c = ((ContainerEvent)e).getChild();
+							if (c instanceof javax.accessibility.Accessible) {
+								AccessibleContext ac = ((javax.accessibility.Accessible)c).getAccessibleContext();
+								AtkWrapper.componentRemoved(ac);
+							}
+						} catch (Exception ex) {
+							ex.printStackTrace();
 						}
 						break;
 					}
@@ -261,7 +268,9 @@ public class AtkWrapper {
 			}
 
 			focusNotify(ctx);
-		} catch (Exception e) {}
+		} catch (Exception e) {
+				e.printStackTrace();
+		}
 	}
 
 	final Toolkit toolkit = Toolkit.getDefaultToolkit();
@@ -547,9 +556,9 @@ public class AtkWrapper {
 					public void dispatchEvent (AWTEvent e) {
 						if (e instanceof KeyEvent) {
 							if (e.getID() == KeyEvent.KEY_PRESSED) {
-								boolean isComsumed =
+								boolean isConsumed =
 									AtkWrapper.dispatchKeyEvent( new AtkKeyEvent((KeyEvent)e) );
-								if (isComsumed) {
+								if (isConsumed) {
 									previousPressConsumed = true;
 									return;
 								}
@@ -558,7 +567,7 @@ public class AtkWrapper {
 									return;
 								}
 							} else if (e.getID() == KeyEvent.KEY_RELEASED) {
-								boolean isConsumed = 
+								boolean isConsumed =
 									AtkWrapper.dispatchKeyEvent( new AtkKeyEvent((KeyEvent)e) );
 								
 								previousPressConsumed = false;
